@@ -44,7 +44,7 @@
 #include <freetds/iconv.h>
 #include <freetds/convert.h>
 #include <freetds/odbc.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #define TDSODBC_BCP
 #include <odbcss.h>
 
@@ -97,12 +97,12 @@ odbc_bcp_init(TDS_DBC *dbc, const ODBC_CHAR *tblname, const ODBC_CHAR *hfile,
 				    dbc, SQLWSTR(tblname->wide), SQLWSTR(hfile->wide), SQLWSTR(errfile->wide), direction);
 			SQLWSTR_FREE();
 		} else {
+#else
+		{
 #endif
-		tdsdump_log(TDS_DBG_FUNC, "bcp_init(%p, %s, %s, %s, %d)\n",
-			    dbc, tblname->mb, hfile->mb, errfile->mb, direction);
-#ifdef ENABLE_ODBC_WIDE
+			tdsdump_log(TDS_DBG_FUNC, "bcp_init(%p, %s, %s, %s, %d)\n",
+				    dbc, (const char*) tblname, (const char*) hfile, (const char*) errfile, direction);
 		}
-#endif
 	}
 	if (!tblname)
 		ODBCBCP_ERROR_RETURN("HY009");
@@ -433,7 +433,7 @@ _tdsodbc_dbconvert(TDS_DBC *dbc, int srctype, const TDS_CHAR * src, SQLLEN src_l
 	TDS_DATETIMEALL dta;
 	TDS_NUMERIC num;
 	SQL_NUMERIC_STRUCT * sql_num;
-	int always_convert = 0;
+	bool always_convert = false;
 
 	assert(src_len >= 0);
 	assert(src != NULL);
@@ -547,7 +547,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 	TDS_TINYINT ti;
 	TDS_SMALLINT si;
 	TDS_INT li;
-	tds_sysdep_int64_type lli;
+	TDS_INT8 lli;
 	TDS_SERVER_TYPE desttype, coltype;
 	SQLLEN col_len;
 	int data_is_null;
@@ -601,7 +601,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 		data_is_null = 1;
 	else if (!data_is_null && bindcol->column_bindlen != SQL_VARLEN_DATA) {
 		if (col_len != SQL_NULL_DATA)
-			col_len = ((tds_sysdep_int64_type)bindcol->column_bindlen < col_len) ? bindcol->column_bindlen : col_len;
+			col_len = ((TDS_INT8)bindcol->column_bindlen < col_len) ? bindcol->column_bindlen : col_len;
 		else
 			col_len = bindcol->column_bindlen;
 	}
@@ -627,7 +627,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 
 	if (data_is_null) {
 		bindcol->bcp_column_data->datalen = 0;
-		bindcol->bcp_column_data->is_null = 1;
+		bindcol->bcp_column_data->is_null = true;
 	} else {
 		if ((converted_data_size =
 		     _tdsodbc_dbconvert(dbc, coltype,
@@ -637,7 +637,7 @@ _bcp_get_col_data(TDSBCPINFO *bcpinfo, TDSCOLUMN *bindcol, int offset)
 		}
 
 		bindcol->bcp_column_data->datalen = converted_data_size;
-		bindcol->bcp_column_data->is_null = 0;
+		bindcol->bcp_column_data->is_null = false;
 	}
 
 	return TDS_SUCCESS;
