@@ -35,6 +35,7 @@
 #include <freetds/utils/string.h>
 #include <freetds/convert.h>
 #include <freetds/enum_cap.h>
+#include <freetds/utils/bjoern-utf8.h>
 #include <odbcss.h>
 
 /**
@@ -315,6 +316,7 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 	if ((flag & 1) != 0) {
 		/* wide characters */
 		const unsigned char *p = (const unsigned char*) s;
+		const unsigned char *const p_end = p + len;
 		SQLWCHAR *dest = (SQLWCHAR*) buffer;
 
 		if (flag&0x20)
@@ -322,30 +324,14 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 #ifndef NDEBUG
 		initial_size = cbBuffer;
 #endif
-		while (len) {
-			unsigned char mask;
-			unsigned u;
-			int l;
+		while (p < p_end) {
+			uint32_t u, state = UTF8_ACCEPT;
 
-			if (!(p[0] & 0x80)) {
-				mask = 0x7f; l = 1;
-			} else if ((p[0] & 0xe0) == 0xc0) {
-				mask = 0x1f; l = 2;
-			} else if ((p[0] & 0xf0) == 0xe0) {
-				mask = 0x0f; l = 3;
-			} else if ((p[0] & 0xf8) == 0xf0) {
-				mask = 0x07; l = 4;
-			} else if ((p[0] & 0xfc) == 0xf8) {
-				mask = 0x03; l = 5;
-			} else {
-				mask = 0x7f; l = 1;
-			}
-			if (len < l)
+			while (decode_utf8(&state, &u, *p++) > UTF8_REJECT && p < p_end)
+				continue;
+			if (state != UTF8_ACCEPT)
 				break;
-			len -= l;
-			u = *p++ & mask;
-			while(--l)
-				u = (u << 6) | (*p++ & 0x3f);
+
 			++out_len;
 			if (SIZEOF_SQLWCHAR == 2 && u >= 0x10000 && u < 0x110000u)
 				++out_len;
@@ -382,35 +368,20 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 	} else if (!dbc || !dbc->mb_conv) {
 		/* to ISO-8859-1 */
 		const unsigned char *p = (const unsigned char*) s;
+		const unsigned char *const p_end = p + len;
 		unsigned char *dest = (unsigned char*) buffer;
 
 #ifndef NDEBUG
 		initial_size = cbBuffer;
 #endif
-		while (len) {
-			unsigned char mask;
-			unsigned u;
-			int l;
+		while (p < p_end) {
+			uint32_t u, state = UTF8_ACCEPT;
 
-			if (!(p[0] & 0x80)) {
-				mask = 0x7f; l = 1;
-			} else if ((p[0] & 0xe0) == 0xc0) {
-				mask = 0x1f; l = 2;
-			} else if ((p[0] & 0xf0) == 0xe0) {
-				mask = 0x0f; l = 3;
-			} else if ((p[0] & 0xf8) == 0xf0) {
-				mask = 0x07; l = 4;
-			} else if ((p[0] & 0xfc) == 0xf8) {
-				mask = 0x03; l = 5;
-			} else {
-				mask = 0x7f; l = 1;
-			}
-			if (len < l)
+			while (decode_utf8(&state, &u, *p++) > UTF8_REJECT && p < p_end)
+				continue;
+			if (state != UTF8_ACCEPT)
 				break;
-			len -= l;
-			u = *p++ & mask;
-			while(--l)
-				u = (u << 6) | (*p++ & 0x3f);
+
 			++out_len;
 			if (!dest)
 				continue;
@@ -753,6 +724,8 @@ odbc_sql_to_c_type_default(int sql_type)
 	case SQL_VARBINARY:
 	case SQL_LONGVARBINARY:
 		return SQL_C_BINARY;
+	case SQL_SS_TABLE:
+		return SQL_C_BINARY;
 		/* TODO interval types */
 	default:
 		return 0;
@@ -870,6 +843,8 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 		return SYBVARBINARY;
 	case SQL_LONGVARBINARY:
 		return SYBIMAGE;
+	case SQL_SS_TABLE:
+		return SYBMSTABLE;
 		/* TODO interval types */
 	default:
 		return TDS_INVALID_TYPE;
@@ -969,6 +944,8 @@ odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_
 	TYPE_NORMAL(SQL_SS_VARIANT) \
 	TYPE_NORMAL(SQL_TYPE_DATE) \
 	TYPE_NORMAL(SQL_TYPE_TIME) \
+\
+	TYPE_NORMAL(SQL_SS_TABLE) \
 \
 	TYPE_VERBOSE_START(SQL_DATETIME) \
 	TYPE_VERBOSE_DATE(SQL_DATETIME, SQL_CODE_TIMESTAMP, SQL_TYPE_TIMESTAMP, SQL_TIMESTAMP) \
@@ -1019,9 +996,13 @@ odbc_set_concise_sql_type(SQLSMALLINT concise_type, struct _drecord * drec, int
 		return SQL_ERROR;
 	}
 	if (!check_only) {
+		if (drec->sql_desc_concise_type == SQL_SS_TABLE)
+			tvp_free((SQLTVP *) drec->sql_desc_data_ptr);
+
 		drec->sql_desc_concise_type = concise_type;
 		drec->sql_desc_type = type;
 		drec->sql_desc_datetime_interval_code = interval_code;
+		drec->sql_desc_data_ptr = NULL;
 
 		switch (drec->sql_desc_type) {
 		case SQL_NUMERIC:
@@ -1146,6 +1127,7 @@ odbc_set_concise_c_type(SQLSMALLINT concise_type, struct _drecord * drec, int ch
 		drec->sql_desc_concise_type = concise_type;
 		drec->sql_desc_type = type;
 		drec->sql_desc_datetime_interval_code = interval_code;
+		drec->sql_desc_data_ptr = NULL;
 
 		switch (drec->sql_desc_type) {
 		case SQL_C_NUMERIC:
