@@ -78,26 +78,43 @@ static const char *parse_numeric(const char *buf, const char *pend,
 
 #define test_alloc(x) {if ((x)==NULL) return TDS_CONVERT_NOMEM;}
 
-#define IS_TINYINT(x) ( 0 <= (x) && (x) <= 0xff )
-#define IS_SMALLINT(x) ( -32768 <= (x) && (x) <= 32767 )
-#define IS_USMALLINT(x) ( 0 <= (x) && (x) <= 65535 )
 /*
+ * Macros for number checks.
+ * They work for signed/unsigned integers and floating point values
+ * (accounting for truncation of any fractional portions of the latter).
+ */
+#define IS_TINYINT(x) ( -1 < (x) && (x) < 0x100 )
+#define IS_SMALLINT(x) ( -32769 < (x) && (x) < 32768 )
+#define IS_USMALLINT(x) ( -1 < (x) && (x) < 65536 )
+
+/*
+ * Macros for integer number checks.
+ * IS_UINT works for both integers and floating point values.
+ *
  * f77: I don't write -2147483648, some compiler seem to have some problem 
  * with this constant although is a valid 32bit value
  */
-#define TDS_INT_MIN (-2147483647l-1l)
-#define TDS_INT_MAX 2147483647l
-#define IS_INT(x) (TDS_INT_MIN <= (x) && (x) <= TDS_INT_MAX)
-
-#define TDS_UINT_MAX 4294967295lu
-#define IS_UINT(x) (0 <= (x) && (x) <= TDS_UINT_MAX)
-
-#define TDS_INT8_MAX ((((TDS_INT8) 0x7fffffffl) << 32) + (TDS_INT8) 0xfffffffflu)
+#define TDS_INT_MIN (-2147483647-1)
+#define TDS_INT_MAX 2147483647
+#define INT_IS_INT(x) (TDS_INT_MIN <= (x) && (x) <= TDS_INT_MAX)
+#define TDS_UINT_MAX 4294967295u
+#define IS_UINT(x) (-1 < (TDS_INT8)(x) && (x) < (TDS_INT8) TDS_UINT_MAX + 1)
 #define TDS_INT8_MIN (-(((TDS_INT8)1)<<62) -(((TDS_INT8)1)<<62))
-#define IS_INT8(x) (TDS_INT8_MIN <= (x) && (x) <= TDS_INT8_MAX)
+#define TDS_INT8_MAX ((((TDS_INT8) 0x7fffffff) << 32) + (TDS_INT8) 0xffffffffu)
 
-#define TDS_UINT8_MAX ((((TDS_UINT8) 0xfffffffflu) << 32) + 0xfffffffflu)
-#define IS_UINT8(x) (0 <= (x) && (x) <= TDS_UINT8_MAX)
+/*
+ * Macros for floating point number checks.
+ * To avoid possible loss of precision, use upper bounds for floating point
+ * values and test MIN - (x) < 1 rather than MIN - 1 < (x) for signed
+ * destination types.
+ */
+#define TDS_INT_UPPER_FLOAT 2147483648.0f
+#define FLOAT_IS_INT(x) (TDS_INT_MIN - (x) < 1.0f && (x) < TDS_INT_UPPER_FLOAT)
+#define TDS_INT8_MIN_FLOAT (-9223372036854775808.0f)
+#define TDS_INT8_UPPER_FLOAT 9223372036854775808.0f
+#define FLOAT_IS_INT8(x) (TDS_INT8_MIN_FLOAT - (x) < 1.0f && (x) < TDS_INT8_UPPER_FLOAT)
+#define TDS_UINT8_UPPER 18446744073709551616.0f
+#define FLOAT_IS_UINT8(x) (-1.0f < (x) && (x) < TDS_UINT8_UPPER)
 
 #define TDS_ISDIGIT(c) ((c) >= '0' && (c) <= '9')
 
@@ -446,7 +463,7 @@ tds_convert_char(const TDS_CHAR * src, TDS_UINT srclen, int desttype, CONV_RESUL
 			cr->m.mny = mymoney;
 			return sizeof(TDS_MONEY);
 		} else {
-			if (!IS_INT(mymoney))
+			if (!INT_IS_INT(mymoney))
 				return TDS_CONVERT_OVERFLOW;
 			cr->m4.mny4 = (TDS_INT) mymoney;
 			return sizeof(TDS_MONEY4);
@@ -713,7 +730,7 @@ tds_convert_int8(const TDS_INT8 *src, int desttype, CONV_RESULT * cr)
 	TDS_CHAR tmp_str[24];
 
 	memcpy(&buf, src, sizeof(buf));
-	if (IS_INT(buf))
+	if (INT_IS_INT(buf))
 		return tds_convert_int((TDS_INT) buf, desttype, cr);
 
 	switch (desttype) {
@@ -790,8 +807,8 @@ tds_convert_uint8(const TDS_UINT8 *src, int desttype, CONV_RESULT * cr)
 	TDS_CHAR tmp_str[24];
 
 	memcpy(&buf, src, sizeof(buf));
-	/* IS_INT does not work here due to unsigned/signed conversions */
-	if (buf <= (TDS_UINT8) TDS_INT_MAX)
+	/* INT_IS_INT does not work here due to unsigned/signed conversions */
+	if (buf <= TDS_INT_MAX)
 		return tds_convert_int((TDS_INT) buf, desttype, cr);
 
 	switch (desttype) {
@@ -815,7 +832,7 @@ tds_convert_uint8(const TDS_UINT8 *src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_UINT);
 		break;
 	case SYBINT8:
-		if (buf > (TDS_UINT8) TDS_INT8_MAX)
+		if (buf > TDS_INT8_MAX)
 			return TDS_CONVERT_OVERFLOW;
 		cr->bi = (TDS_INT8) buf;
 		return sizeof(TDS_INT8);
@@ -1113,7 +1130,7 @@ tds_convert_money4(const TDSCONTEXT * tds_ctx, const TDS_MONEY4 * src, int destt
 		break;
 	case SYBUINT8:
 		dollars = mny.mny4 / 10000;
-		if (!IS_UINT8(dollars))
+		if (dollars < 0)
 			return TDS_CONVERT_OVERFLOW;
 		cr->ubi = dollars;
 		return sizeof(TDS_UINT8);
@@ -1195,7 +1212,7 @@ tds_convert_money(const TDSCONTEXT * tds_ctx, const TDS_MONEY * src, int desttyp
 		break;
 	case SYBINT4:
 		dollars = mymoney / 10000;
-		if (!IS_INT(dollars))
+		if (!INT_IS_INT(dollars))
 			return TDS_CONVERT_OVERFLOW;
 		cr->i = (TDS_INT) dollars;
 		return sizeof(TDS_INT);
@@ -1232,7 +1249,7 @@ tds_convert_money(const TDSCONTEXT * tds_ctx, const TDS_MONEY * src, int desttyp
 		return sizeof(TDS_REAL);
 		break;
 	case SYBMONEY4:
-		if (!IS_INT(mymoney))
+		if (!INT_IS_INT(mymoney))
 			return TDS_CONVERT_OVERFLOW;
 		cr->m4.mny4 = (TDS_INT) mymoney;
 		return sizeof(TDS_MONEY4);
@@ -1262,20 +1279,24 @@ static TDS_INT
 tds_convert_datetimeall(const TDSCONTEXT * tds_ctx, int srctype, const TDS_DATETIMEALL * dta, int desttype, CONV_RESULT * cr)
 {
 	char whole_date_string[64];
+	const char *datetime_fmt;
 	TDSDATEREC when;
 
 	switch (desttype) {
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		tds_datecrack(srctype, dta, &when);
-		tds_strftime(whole_date_string, sizeof(whole_date_string), tds_ctx->locale->date_fmt, &when, 
+		datetime_fmt = tds_ctx->locale->datetime_fmt;
+		if (srctype == SYBMSDATE && tds_ctx->locale->date_fmt)
+			datetime_fmt = tds_ctx->locale->date_fmt;
+		if (srctype == SYBMSTIME && tds_ctx->locale->time_fmt)
+			datetime_fmt = tds_ctx->locale->time_fmt;
+		tds_strftime(whole_date_string, sizeof(whole_date_string), datetime_fmt, &when,
 		             dta->time_prec);
 
 		return string_to_result(desttype, whole_date_string, cr);
 	case SYBDATETIME:
-		if (!IS_INT(dta->date))
-			return TDS_CONVERT_OVERFLOW;
-		cr->dt.dtdays = (TDS_INT) dta->date;
+		cr->dt.dtdays = dta->date;
 		cr->dt.dttime = (TDS_INT) ((dta->time * 3u + 50000u) / 100000u);
 		return sizeof(TDS_DATETIME);
 	case SYBDATETIME4:
@@ -1291,9 +1312,7 @@ tds_convert_datetimeall(const TDSCONTEXT * tds_ctx, int srctype, const TDS_DATET
 		cr->dta = *dta;
 		return sizeof(TDS_DATETIMEALL);
 	case SYBDATE:
-		if (!IS_INT(dta->date))
-			return TDS_CONVERT_OVERFLOW;
-		cr->date = (TDS_INT) dta->date;
+		cr->date = dta->date;
 		return sizeof(TDS_DATE);
 	case SYBTIME:
 		cr->time = (TDS_INT) ((dta->time * 3u + 50000u) / 100000u);
@@ -1337,7 +1356,7 @@ tds_convert_datetime(const TDSCONTEXT * tds_ctx, const TDS_DATETIME * dt, int de
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		tds_datecrack(SYBDATETIME, dt, &when);
-		tds_strftime(whole_date_string, sizeof(whole_date_string), tds_ctx->locale->date_fmt, &when, 3);
+		tds_strftime(whole_date_string, sizeof(whole_date_string), tds_ctx->locale->datetime_fmt, &when, 3);
 
 		return string_to_result(desttype, whole_date_string, cr);
 	case SYBDATETIME:
@@ -1421,33 +1440,37 @@ tds_convert_datetime4(const TDSCONTEXT * tds_ctx, const TDS_DATETIME4 * dt4, int
 static TDS_INT
 tds_convert_time(const TDSCONTEXT * tds_ctx, const TDS_TIME * time, int desttype, CONV_RESULT * cr)
 {
-	TDS_DATETIME dt;
+	TDS_DATETIMEALL dta;
 
 	if (desttype == SYBTIME) {
 		cr->time = *time;
 		return sizeof(TDS_TIME);
 	}
 
-	/* convert to DATETIME and use tds_convert_datetime */
-	dt.dtdays = 0;
-	dt.dttime = *time;
-	return tds_convert_datetime(tds_ctx, &dt, desttype, 0, cr);
+	/* convert to DATETIMEALL and use tds_convert_datetimeall */
+	memset(&dta, 0, sizeof(dta));
+	dta.has_time = 1;
+	dta.time_prec = 3;
+	dta.time = ((TDS_UINT8) *time * 100000 + 2) / 3;
+	dta.time = (((TDS_UINT8) *time) * 20 + 3) / 6 * 10000;
+	return tds_convert_datetimeall(tds_ctx, SYBMSTIME, &dta, desttype, cr);
 }
 
 static TDS_INT
 tds_convert_date(const TDSCONTEXT * tds_ctx, const TDS_DATE * date, int desttype, CONV_RESULT * cr)
 {
-	TDS_DATETIME dt;
+	TDS_DATETIMEALL dta;
 
 	if (desttype == SYBDATE) {
 		cr->date = *date;
 		return sizeof(TDS_DATE);
 	}
 
-	/* convert to DATETIME and use tds_convert_datetime */
-	dt.dtdays = *date;
-	dt.dttime = 0;
-	return tds_convert_datetime(tds_ctx, &dt, desttype, 0, cr);
+	/* convert to DATETIMEALL and use tds_convert_datetimeall */
+	memset(&dta, 0, sizeof(dta));
+	dta.has_date = 1;
+	dta.date = *date;
+	return tds_convert_datetimeall(tds_ctx, SYBMSDATE, &dta, desttype, cr);
 }
 
 static TDS_INT
@@ -1497,7 +1520,7 @@ tds_convert_real(const TDS_REAL* src, int desttype, CONV_RESULT * cr)
 {
 	TDS_REAL the_value;
 
-/* FIXME how many big should be this buffer ?? */
+	/* FIXME how big should be this buffer ?? */
 	char tmp_str[128];
 	TDS_INT mymoney4;
 	TDS_INT8 mymoney;
@@ -1530,7 +1553,7 @@ tds_convert_real(const TDS_REAL* src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_USMALLINT);
 		break;
 	case SYBINT4:
-		if (!IS_INT(the_value))
+		if (!FLOAT_IS_INT(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->i = (TDS_INT) the_value;
 		return sizeof(TDS_INT);
@@ -1542,13 +1565,13 @@ tds_convert_real(const TDS_REAL* src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_UINT);
 		break;
 	case SYBINT8:
-		if (the_value > (TDS_REAL) TDS_INT8_MAX || the_value < (TDS_REAL) TDS_INT8_MIN)
+		if (!FLOAT_IS_INT8(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->bi = (TDS_INT8) the_value;
 		return sizeof(TDS_INT8);
 		break;
 	case SYBUINT8:
-		if (the_value > (TDS_REAL) TDS_UINT8_MAX || the_value < 0)
+		if (!FLOAT_IS_UINT8(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->ubi = (TDS_UINT8) the_value;
 		return sizeof(TDS_UINT8);
@@ -1639,7 +1662,7 @@ tds_convert_flt8(const TDS_FLOAT* src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_USMALLINT);
 		break;
 	case SYBINT4:
-		if (!IS_INT(the_value))
+		if (!FLOAT_IS_INT(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->i = (TDS_INT) the_value;
 		return sizeof(TDS_INT);
@@ -1651,13 +1674,13 @@ tds_convert_flt8(const TDS_FLOAT* src, int desttype, CONV_RESULT * cr)
 		return sizeof(TDS_UINT);
 		break;
 	case SYBINT8:
-		if (the_value > (TDS_FLOAT) TDS_INT8_MAX || the_value < (TDS_FLOAT) TDS_INT8_MIN)
+		if (!FLOAT_IS_INT8(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->bi = (TDS_INT8) the_value;
 		return sizeof(TDS_INT8);
 		break;
 	case SYBUINT8:
-		if (the_value > (TDS_FLOAT) TDS_UINT8_MAX || the_value < 0)
+		if (!FLOAT_IS_UINT8(the_value))
 			return TDS_CONVERT_OVERFLOW;
 		cr->ubi = (TDS_UINT8) the_value;
 		return sizeof(TDS_UINT8);
@@ -1882,7 +1905,7 @@ tds_convert(const TDSCONTEXT *tds_ctx, int srctype, const void *src, TDS_UINT sr
 	assert(srclen >= 0 && srclen <= 2147483647u);
 
 	if (srctype == SYBVARIANT) {
-		TDSVARIANT *v = (TDSVARIANT*) src;
+		const TDSVARIANT *v = (const TDSVARIANT*) src;
 		srctype = v->type;
 		src = v->data;
 		srclen = v->data_len;
@@ -1971,6 +1994,7 @@ tds_convert(const TDSCONTEXT *tds_ctx, int srctype, const void *src, TDS_UINT sr
 		break;
 	case SYBNVARCHAR:
 	case SYBNTEXT:
+	case SYBMSTABLE:
 	default:
 		return TDS_CONVERT_NOAVAIL;
 		break;
@@ -2982,7 +3006,7 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 
 	size_t length;
 	char *our_format;
-	char *pz = NULL;
+	char *pz;
 	bool z_found = false;
 	
 	assert(buf);
@@ -3008,7 +3032,7 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 #endif
 
 	/* more characters are required because we replace %z with up to 7 digits */
-	our_format = tds_new(char, strlen(format) + 1 + 5);
+	our_format = tds_new(char, strlen(format) + 1 + 5 + 1);
 	if (!our_format)
 		return 0;
 
@@ -3020,6 +3044,8 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 
 		switch (*pz) {
 		case 0:
+			*pz++ = '%';
+			*pz = 0;
 			continue;
 		case '%':
 			/* escaped, do not treat as format on next iteration */
