@@ -207,9 +207,6 @@
 #define USE_ICONV (tds->conn->use_iconv)
 
 static const TDSCOLUMNFUNCS *tds_get_column_funcs(TDSCONNECTION *conn, int type);
-#ifdef WORDS_BIGENDIAN
-static void tds_swap_datatype(int coltype, void *b);
-#endif
 static void tds_swap_numeric(TDS_NUMERIC *num);
 
 #undef MIN
@@ -401,7 +398,11 @@ tds_generic_get_info(TDSSOCKET *tds, TDSCOLUMN *col)
 		col->column_size = tds_get_smallint(tds);
 		/* under TDS7.2 this means ?var???(MAX) */
 		if (col->column_size < 0 && IS_TDS72_PLUS(tds->conn)) {
-			col->column_size = 0x3ffffffflu;
+			if (is_char_type(col->column_type))
+				col->column_size = 0x3ffffffflu;
+			else
+				col->column_size = 0x7ffffffflu;
+
 			col->column_varint_size = 8;
 		}
 		if (col->column_size < 0)
@@ -442,7 +443,7 @@ tds_generic_get_info(TDSSOCKET *tds, TDSCOLUMN *col)
 	} else if (IS_TDS72_PLUS(tds->conn) && col->on_server.column_type == SYBMSXML) {
 		unsigned char has_schema = tds_get_byte(tds);
 		if (has_schema) {
-			/* discard schema information */
+			/* discard schema informations */
 			tds_get_string(tds, tds_get_byte(tds), NULL, 0);        /* dbname */
 			tds_get_string(tds, tds_get_byte(tds), NULL, 0);        /* schema owner */
 			tds_get_string(tds, tds_get_usmallint(tds), NULL, 0);    /* schema collection */
@@ -605,6 +606,8 @@ tds_variant_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 			TDS_ZERO_FREE(v->data);
 		v->data_len = sizeof(TDS_NUMERIC);
 		num = tds_new0(TDS_NUMERIC, 1);
+		if (!num)
+			goto error_memory;
 		v->data = (TDS_CHAR *) num;
 		num->precision = tds_get_byte(tds);
 		num->scale     = tds_get_byte(tds);
@@ -695,6 +698,7 @@ tds_variant_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 	return TDS_SUCCESS;
 
 error_type:
+error_memory:
 	tds_get_n(tds, NULL, colsize);
 	return TDS_FAIL;
 }
@@ -725,10 +729,10 @@ tds_generic_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 		if (len == 16) {	/*  Jeff's hack */
 			tds_get_n(tds, blob->textptr, 16);
 			tds_get_n(tds, blob->timestamp, 8);
-			blob->valid_ptr = 1;
+			blob->valid_ptr = true;
 			if (IS_TDS72_PLUS(tds->conn) &&
 			    memcmp(blob->textptr, "dummy textptr\0\0",16) == 0)
-				blob->valid_ptr = 0;
+				blob->valid_ptr = false;
 			colsize = tds_get_int(tds);
 		} else {
 			colsize = -1;
@@ -1472,6 +1476,107 @@ tds_sybbigtime_put(TDSSOCKET *tds, TDSCOLUMN *col, int bcp7)
 	return TDS_SUCCESS;
 }
 
+TDSRET
+tds_mstabletype_get_info(TDSSOCKET *tds, TDSCOLUMN *col)
+{
+	/* Table type is strictly only an input variable */
+	return TDS_FAIL;
+}
+
+TDS_INT
+tds_mstabletype_row_len(TDSCOLUMN *col)
+{
+	return sizeof(TDS_TVP);
+}
+
+TDSRET
+tds_mstabletype_get(TDSSOCKET *tds, TDSCOLUMN *col)
+{
+	/* Table type is strictly only an input variable */
+	return TDS_FAIL;
+}
+
+TDSRET
+tds_mstabletype_put_info(TDSSOCKET *tds, TDSCOLUMN *col)
+{
+	TDS_TVP *table = (TDS_TVP *) col->column_data;
+	TDSFREEZE current_freeze[1];
+	size_t written;
+
+	/* TVP_TYPENAME */
+	tds_put_byte(tds, 0); /* Empty DB name */
+
+	tds_freeze(tds, current_freeze, 1);
+	tds_put_string(tds, table->schema, -1);
+	written = tds_freeze_written(current_freeze) / 2;
+	tds_freeze_close_len(current_freeze, written);
+
+	tds_freeze(tds, current_freeze, 1);
+	tds_put_string(tds, table->name, -1);
+	written = tds_freeze_written(current_freeze) / 2;
+	tds_freeze_close_len(current_freeze, written);
+
+	return TDS_SUCCESS;
+}
+
+TDSRET
+tds_mstabletype_put(TDSSOCKET *tds, TDSCOLUMN *col, int bcp7)
+{
+	TDS_TVP *table = (TDS_TVP *) col->column_data;
+	TDSPARAMINFO *params;
+	TDSCOLUMN *tds_col;
+	TDS_TVP_ROW *row;
+	TDSRET ret;
+	int i;
+	TDS_USMALLINT num_cols = table->metadata ? table->metadata->num_cols : 0;
+
+	/* COL_METADATA */
+	if (num_cols == 0)
+		tds_put_smallint(tds, 0xffff); /* TVP_NULL_TOKEN */
+	else {
+		tds_put_smallint(tds, num_cols);
+
+		params = table->metadata;
+		for (i = 0; i < num_cols; i++) {
+			tds_col = params->columns[i];
+
+			/* UserType*/
+			tds_put_int(tds, tds_col->column_usertype);
+			/* Flags */
+			tds_put_smallint(tds, tds_col->column_flags);
+			/* TYPE_INFO */
+			tds_put_byte(tds, tds_col->on_server.column_type);
+			ret = tds_col->funcs->put_info(tds, tds_col);
+			if (TDS_FAILED(ret))
+				return ret;
+
+			/* ColName - Empty string */
+			tds_put_byte(tds, 0x00);
+		}
+	}
+
+	/* TVP_END_TOKEN */
+	tds_put_byte(tds, 0x00);
+
+	for (row = table->row; row != NULL; row = row->next) {
+		/* TVP_ROW_TOKEN */
+		tds_put_byte(tds, 0x01);
+
+		params = row->params;
+		for (i = 0; i < num_cols; i++) {
+			tds_col = params->columns[i];
+			ret = tds_col->funcs->put_data(tds, tds_col, 0);
+			if (TDS_FAILED(ret))
+				return ret;
+		}
+	}
+
+	/* TVP_END_TOKEN */
+	tds_put_byte(tds, 0x00);
+
+	return TDS_SUCCESS;
+}
+
 TDSRET
 tds_invalid_get_info(TDSSOCKET * tds, TDSCOLUMN * col)
 {
@@ -1526,6 +1631,12 @@ tds_sybbigtime_check(const TDSCOLUMN *col)
 	return 1;
 }
 
+int
+tds_mstabletype_check(const TDSCOLUMN *col)
+{
+	return 0;
+}
+
 int
 tds_clrudt_check(const TDSCOLUMN *col)
 {
@@ -1594,6 +1705,7 @@ TDS_DECLARE_FUNCS(msdatetime);
 TDS_DECLARE_FUNCS(clrudt);
 TDS_DECLARE_FUNCS(sybbigtime);
 TDS_DECLARE_FUNCS(invalid);
+TDS_DECLARE_FUNCS(mstabletype);
 #include <freetds/popvis.h>
 
 static const TDSCOLUMNFUNCS *
@@ -1617,13 +1729,15 @@ tds_get_column_funcs(TDSCONNECTION *conn, int type)
 	case SYB5BIGTIME:
 	case SYB5BIGDATETIME:
 		return &tds_sybbigtime_funcs;
+	case SYBMSTABLE:
+		return &tds_mstabletype_funcs;
 	}
 	return &tds_generic_funcs;
 }
 #include "tds_types.h"
 
 #ifdef WORDS_BIGENDIAN
-static void
+void
 tds_swap_datatype(int coltype, void *b)
 {
 	unsigned char *buf = (unsigned char *) b;
@@ -1632,12 +1746,14 @@ tds_swap_datatype(int coltype, void *b)
 	case SYBDATETIME4:
 		tds_swap_bytes(&buf[2], 2);
 	case SYBINT2:
+	case SYBUINT2:
 		tds_swap_bytes(buf, 2);
 		break;
 	case SYBMONEY:
 	case SYBDATETIME:
 		tds_swap_bytes(&buf[4], 4);
 	case SYBINT4:
+	case SYBUINT4:
 	case SYBMONEY4:
 	case SYBREAL:
 	case SYBDATE:
@@ -1645,6 +1761,7 @@ tds_swap_datatype(int coltype, void *b)
 		tds_swap_bytes(buf, 4);
 		break;
 	case SYBINT8:
+	case SYBUINT8:
 	case SYBFLT8:
 	case SYB5BIGTIME:
 	case SYB5BIGDATETIME:
