@@ -582,7 +582,7 @@ tds_ssl_deinit(TDSCONNECTION *conn)
 	conn->encrypt_single_packet = 0;
 }
 
-#else
+#else /* !HAVE_GNUTLS */
 static long
 tds_ssl_ctrl_login(BIO *b, int cmd, long num, void *ptr)
 {
@@ -826,18 +826,25 @@ tds_check_wildcard_test(void)
 static int
 check_name_match(ASN1_STRING *name, const char *hostname)
 {
-	char *name_utf8 = NULL;
+	char *name_utf8 = NULL, *tmp_name;
 	int ret, name_len;
 
 	name_len = ASN1_STRING_to_UTF8((unsigned char **) &name_utf8, name);
 	if (name_len < 0)
 		return 0;
 
+	tmp_name = tds_strndup(name_utf8, name_len);
+	OPENSSL_free(name_utf8);
+	if (!tmp_name)
+		return 0;
+
+	name_utf8 = tmp_name;
+
 	tdsdump_log(TDS_DBG_INFO1, "Got name %s\n", name_utf8);
 	ret = 0;
 	if (strlen(name_utf8) == name_len && check_wildcard(name_utf8, hostname))
 		ret = 1;
-	OPENSSL_free(name_utf8);
+	free(name_utf8);
 	return ret;
 }
 
@@ -862,13 +869,11 @@ check_alt_names(X509 *cert, const char *hostname)
 		ret = inet_pton(AF_INET, hostname, &ip.v4);
 	}
 	if (ret == 0)
-		return -1;
-
-	ret = -1;
+		ip_size = 0;
 
 	alt_names = X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
 	if (!alt_names)
-		return ret;
+		return -1;
 
 	num = sk_GENERAL_NAME_num(alt_names);
 	tdsdump_log(TDS_DBG_INFO1, "Alt names number %d\n", num);
@@ -884,11 +889,9 @@ check_alt_names(X509 *cert, const char *hostname)
 		altlen = (size_t) ASN1_STRING_length(name->d.ia5);
 
 		if (name->type == GEN_DNS && ip_size == 0) {
-			ret = 0;
 			if (!check_name_match(name->d.dNSName, hostname))
 				continue;
 		} else if (name->type == GEN_IPADD && ip_size != 0) {
-			ret = 0;
 			if (altlen != ip_size || memcmp(altptr, &ip, altlen) != 0)
 				continue;
 		} else {
@@ -899,7 +902,7 @@ check_alt_names(X509 *cert, const char *hostname)
 		return 1;
 	}
 	sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
-	return ret;
+	return -1;
 }
 
 static int
